<?php declare(strict_types=1);

namespace Convo\Proto;

use Convo\Core\DataItemNotFoundException;
use Psr\Http\Server\RequestHandlerInterface;

class OAuthRestHandler implements RequestHandlerInterface
{

    /**
     * @var \Convo\Proto\ServiceUserDao
     */
    private $_serviceUserDao;

    /**
     * @var \Convo\Core\Util\IHttpFactory
     */
    private $_httpFactory;

    /**
     * @var \Psr\Log\LoggerInterface
     */
    private $_logger;

    public function __construct( $httpFactory, $logger, $serviceUserDao)
    {
    	$this->_logger				= 	$logger;
    	$this->_httpFactory			= 	$httpFactory;
        $this->_serviceUserDao		=	$serviceUserDao;
    }

    public function handle(\Psr\Http\Message\ServerRequestInterface $request): \Psr\Http\Message\ResponseInterface
    {
    	$info			=	new \Convo\Core\Rest\RequestInfo( $request);

    	if ( $info->get() && $route = $info->route( 'oauth/{type}'))
    	{
    		return $this->_handleOauthPathTypeGet( $request, $route->get('type'));
    	}

    	if ( $info->post() && $route = $info->route( 'token/{type}'))
    	{
    		return $this->_handleTokenPathTypePost($request, $route->get('type'));
    	}

    	throw new \Convo\Core\Rest\NotFoundException( 'Could not map ['.$info.']');
    }

    private function _handleOauthPathTypeGet(\Psr\Http\Message\ServerRequestInterface $request, $type)
    {
        $params			=	$request->getQueryParams();

        $user_id        =   $params['user_id'] ?? null;

        if (!$user_id) {
            return $this->_httpFactory->buildResponse([
                "message" => "Missing user_id query parameter."
            ], 400);
        }

        $state 			=	$params['state'] ?? null;
        $client_id 		=	$params['client_id'] ?? null;
        $response_type	=	$params['response_type'] ?? null;
        $scope			=	$params['scope'] ?? null;
        $redirect_uri	=	$params['redirect_uri'] ?? null;

        $this->_logger->debug("Got params [$state][$client_id][$response_type][$scope][$redirect_uri]");

        try {
            $user = $this->_serviceUserDao->getUserById($user_id);

            $code = $this->_generateAuthCodeForUser($user);

            if (isset($user['authCode'][$type]['code']) &&
                    $user['authCode'][$type]['code'] === $code &&
                    $user['authCode'][$type]['redeemed'] === true)
            {
                return $this->_httpFactory->buildResponse(['message' => 'Code has already been redeemed.'], 400);
            }

            $this->_serviceUserDao->updateUser($user['userId'], [
                'authCode' => [
                    $type => [
                        'code' => $code,
                        'redeemed' => false
                    ]
                ]
            ]);

            return $this->_httpFactory->buildResponse([], 302, [
                'Location' => $redirect_uri."?state={$state}&code={$code}"
            ]);
        } catch (DataItemNotFoundException $e) {
            return $this->_httpFactory->buildResponse([], 401);
        }
    }

    private function _handleTokenPathTypePost(\Psr\Http\Message\ServerRequestInterface $request, $type)
    {
        $json = $request->getParsedBody();

        $this->_logger->debug('Got JSON ['.print_r($json, true).']');

        $grant_type = $json['grant_type'] ?? null;

        if (!$grant_type) {
            throw new \Exception('Missing grant type.');
        }

        switch ($grant_type) {
            case 'refresh_token':
                $refresh_token = $json['refresh_token'];
                return $this->_refreshToken($refresh_token, $type);
            case 'authorization_code':
                $auth_code = $json['code'];
                return $this->_redeemCodeForToken($auth_code, $type);
            default:
                return $this->_httpFactory->buildResponse([
                    'error' => 'invalid_grant'
                ], 400);
        }
    }

    private function _refreshToken($refreshToken, $type) {
        $expires = $type === 'google' ? 'expires_in' : 'expires';

        $user = $this->_serviceUserDao->getUserByRefreshToken($refreshToken, $type);

        $auth_token = bin2hex(random_bytes(64));

        $token_data = [
            $type => [
                'access_token' => $auth_token,
                'refresh_token' => $refreshToken,
                'token_type' => 'bearer',
                $expires => 3600
            ]
        ];

        $this->_serviceUserDao->updateUser($user['userId'], [
            'accessToken' => $token_data
        ]);

        return $this->_httpFactory->buildResponse($token_data[$type]);
    }

    private function _redeemCodeForToken($code, $type) {
        $expires = $type === 'google' ? 'expires_in' : 'expires';

        try {
            $user = $this->_serviceUserDao->getUserByAuthCode($code, $type);

            // todo: mix in some user data so that this isn't completely random?
            $auth_token = bin2hex(random_bytes(64));
            $refresh_token = bin2hex(random_bytes(16));

            $token_data = [
                $type => [
                    'access_token' => $auth_token,
                    'refresh_token' => $refresh_token,
                    'token_type' => 'bearer',
                    $expires => 3600
                ]
            ];

            $this->_serviceUserDao->updateUser($user['userId'], [
                'authCode' => [
                    $type => ['redeemed' => true]
                ],
                'accessToken' => $token_data
            ]);

            return $this->_httpFactory->buildResponse($token_data[$type]);
        } catch (DataItemNotFoundException $e) {
            return $this->_httpFactory->buildResponse(['message' => 'Auth code not found.'], 401);
        }
    }

    private function _generateAuthCodeForUser($user)
    {
        $data = $user['userId'].$user['firstName'].$user['lastName'].bin2hex(random_bytes(16));

        return hash("haval128,3", $data);
    }

    public function __toString()
    {
        return get_class($this).'[]';
    }
}
